import * as jose from "jose";
export declare type Key = jose.KeyLike | Uint8Array;
export declare const KEY_TYPES: readonly ["EC", "RSA", "oct"];
export declare const KEY_USES: readonly ["sig", "enc"];
export declare const KEY_OPERATIONS: readonly ["sign", "verify", "encrypt", "decrypt", "wrapKey", "unwrapKey", "deriveKey", "deriveBits"];
export declare type KeyType = typeof KEY_TYPES[number];
export declare type KeyUse = typeof KEY_USES[number];
export declare type KeyOperation = typeof KEY_OPERATIONS[number];
export declare const PUBLIC_KEY_OPERATIONS: readonly ["verify"];
export declare const PRIVATE_KEY_OPERATIONS: readonly ["sign", "decrypt", "unwrapKey", "deriveKey", "deriveBits"];
export declare const SYMMETRIC_KEYS_OPERATIONS: readonly ["sign", "verify", "encrypt", "decrypt"];
export declare type PublicKeyOperation = typeof PUBLIC_KEY_OPERATIONS[number];
export declare type PrivateKeyOperation = typeof PRIVATE_KEY_OPERATIONS[number];
export declare type SymmetricKeyOperation = typeof SYMMETRIC_KEYS_OPERATIONS[number];
export declare const EC_ALGORITHMS: readonly ["ES256", "ES384", "ES512"];
export declare const RSA_ALGORITHMS: readonly ["RS256", "RS384", "RS512"];
export declare const HMAC_ALGORITHMS: readonly ["HS256", "HS384", "HS512"];
export declare const ALGORITHMS: readonly ["ES256", "ES384", "ES512", "RS256", "RS384", "RS512", "HS256", "HS384", "HS512"];
export declare type ECAlgorithm = typeof EC_ALGORITHMS[number];
export declare type RSAAlgorithm = typeof RSA_ALGORITHMS[number];
export declare type HMACAlgorithm = typeof HMAC_ALGORITHMS[number];
export declare type Algorithm = typeof ALGORITHMS[number];
export declare type JWK = JWK_EC | JWK_RSA | JWK_OCT;
export interface JWKBase {
    kty: KeyType;
    use: KeyUse;
    key_ops: KeyOperation[];
    alg?: Algorithm;
    kid?: string;
    x5u?: string;
    x5c?: string[];
    x5t?: string;
    "x5t#S256"?: string;
}
export declare type JWK_EC = JWK_EC_Private | JWK_EC_Public;
export declare type JWK_EC_Public = JWK_EC_Public_Sign | JWK_EC_Public_Encrypt;
export declare type JWK_EC_Private = JWK_EC_Private_Sign | JWK_EC_Private_Encrypt;
export interface JWK_EC_Public_Sign extends JWKBase {
    kty: "EC";
    use: "sig";
    key_ops: "verify"[];
    alg: ECAlgorithm;
    crv: "P-256" | "P-384" | "P-521";
    x: string;
    y: string;
}
export interface JWK_EC_Public_Encrypt extends JWKBase {
    kty: "EC";
    use: "enc";
    key_ops: ("encrypt" | "wrapKey")[];
    alg: ECAlgorithm;
    crv: "P-256" | "P-384" | "P-521";
    x: string;
    y: string;
}
export interface JWK_EC_Private_Sign extends JWKBase {
    kty: "EC";
    use: "sig";
    key_ops: ("sign" | "verify")[];
    alg: ECAlgorithm;
    crv: "P-256" | "P-384" | "P-521";
    x: string;
    y: string;
    d: string;
}
export interface JWK_EC_Private_Encrypt extends JWKBase {
    kty: "EC";
    use: "enc";
    key_ops: ("decrypt" | "unwrapKey")[];
    alg: ECAlgorithm;
    crv: "P-256" | "P-384" | "P-521";
    x: string;
    y: string;
    d: string;
}
export declare type JWK_RSA = JWK_RSA_Private | JWK_RSA_Public;
export declare type JWK_RSA_Public = JWK_RSA_Public_Sign | JWK_RSA_Public_Encrypt;
export declare type JWK_RSA_Private = JWK_RSA_Private_Sign | JWK_RSA_Private_Encrypt;
export interface JWK_RSA_Public_Sign extends JWKBase {
    kty: "RSA";
    use: "sig";
    key_ops: "verify"[];
    alg: RSAAlgorithm;
    n: string;
    e: string;
}
export interface JWK_RSA_Public_Encrypt extends JWKBase {
    kty: "RSA";
    use: "enc";
    key_ops: ("encrypt" | "wrapKey")[];
    alg: RSAAlgorithm;
    n: string;
    e: string;
}
export interface JWK_RSA_Private_Sign extends JWKBase {
    readonly kty: "RSA";
    readonly use: "sig";
    readonly key_ops: ("sign" | "verify")[];
    readonly alg: RSAAlgorithm;
    readonly n: string;
    readonly e: string;
    readonly d: string;
    readonly p: string;
    readonly q: string;
    readonly dp: string;
    readonly dq: string;
    readonly qi: string;
}
export interface JWK_RSA_Private_Encrypt extends JWKBase {
    readonly kty: "RSA";
    readonly use: "enc";
    readonly key_ops: ("decrypt" | "unwrapKey")[];
    readonly alg: RSAAlgorithm;
    readonly n: string;
    readonly e: string;
    readonly d: string;
    readonly p: string;
    readonly q: string;
    readonly dp: string;
    readonly dq: string;
    readonly qi: string;
}
export interface JWK_OCT extends JWKBase {
    kty: "oct";
    k: string;
}
export interface JWTPayloadOptions {
    readonly iss: string | undefined;
    readonly sub: string | undefined;
    readonly aud: string;
    readonly iat: Date | undefined;
    readonly nbf: Date | undefined;
    readonly jti: string | undefined;
}
export interface JWTPayloadOptionsDefault extends JWTPayloadOptions {
}
export interface JWTJSONForm<O extends JWTPayloadOptions, P extends {} | undefined, H extends {} | undefined, SIG extends boolean> {
    header: H extends {} ? H & JWTHeader : JWTHeader;
    payload: P extends {} ? P & JWTPayloadJSONForm<O> : JWTPayloadJSONForm<O>;
    signature: SIG extends true ? string : undefined;
}
export interface JWTHeader {
    readonly alg: Algorithm;
    readonly typ: "JWT";
}
export interface JWTPayload<O extends JWTPayloadOptions = JWTPayloadOptionsDefault> {
    readonly iss: O["iss"];
    readonly sub: O["sub"];
    readonly aud: O["aud"][];
    readonly exp: Date;
    readonly iat: O["iat"];
    readonly nbf: O["nbf"];
    readonly jti: O["jti"];
}
export interface JWTPayloadJSONForm<O extends JWTPayloadOptions = JWTPayloadOptionsDefault> {
    readonly iss: O["iss"];
    readonly sub: O["sub"];
    readonly aud: O["aud"][];
    readonly exp: number;
    readonly iat: O["iat"] extends Date ? number : undefined;
    readonly nbf: O["nbf"] extends Date ? number : undefined;
    readonly jti: O["jti"];
}
export declare const TIME_UNITS: readonly ["seconds", "milliseconds", "mircoseconds"];
export declare type TimeUnit = typeof TIME_UNITS[number];
export declare const TimeUnitsMSMultiplierMap: Record<TimeUnit, number>;
export interface JWTConstructorOptions<O extends JWTPayloadOptions, P extends {} | undefined, H extends {} | undefined, SIG extends boolean = true> {
    readonly algorithm: Algorithm;
    readonly issuer: O["iss"];
    readonly subject: O["sub"];
    readonly audience: O["aud"][];
    readonly expiration: Date;
    readonly notBefore: O["nbf"];
    readonly issuedAt: O["iat"];
    readonly jwtID: O["jti"];
    readonly signature: SIG extends true ? string : undefined;
    readonly additionalHeaders: H;
    readonly additionalPayload: P;
}
export interface JWTHeaderConstructorOptions<A extends {} | undefined> {
    readonly alg: Algorithm;
    readonly additionalHeaders: A;
}
export interface JWTPayloadConstructorOptions<A extends {} | undefined, O extends JWTPayloadOptions = JWTPayloadOptionsDefault> {
    readonly iss: O["iss"];
    readonly sub: O["sub"];
    readonly aud: O["aud"][];
    readonly exp: Date;
    readonly iat: O["iat"];
    readonly nbf: O["nbf"];
    readonly jti: O["jti"];
    readonly additionalPayload: A;
}
export interface IssuerOptions<O extends JWTPayloadOptions, P extends {} | undefined, H extends {} | undefined> {
    readonly additionalHeaders: H;
    readonly algorithm: Algorithm;
    readonly validTimeMs: number;
    readonly validityDelayMs: number;
    readonly nameOrUrl: O["iss"];
    readonly generateJWTID: () => O["jti"];
    readonly defaultPayload: P;
}
export interface IssueTokenOptions<O extends JWTPayloadOptions, P extends {} | undefined, H extends {} | undefined> {
    readonly audience: O["aud"][] | O["aud"] | undefined;
    readonly validTimeMsOverride?: number;
    readonly validityDelayMsOverride?: number;
    readonly subject: O["sub"];
    readonly additionalPayload: Partial<P>;
}
export interface Parser<T> {
    parseToken(token: string): Promise<T>;
}
export interface Verifier<T> extends Parser<T> {
    verifyToken(token: T): Promise<boolean>;
    parseThenVerifyToken(token: string): Promise<T | false>;
}
export interface Issuer<T, O> extends Verifier<T> {
    issueToken(options: O): Promise<T>;
}
//# sourceMappingURL=types.d.ts.map